# Copyright 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the &quot;License&quot;);
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an &quot;AS IS&quot; BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This class is auto generated by the swagger code generator program.
# https://github.com/swagger-api/swagger-codegen.git
# Do not edit the class manually.

defmodule GoogleApi.AndroidDeviceProvisioning.V1.Model.Configuration do
  @moduledoc """
  A configuration collects the provisioning options for Android devices. Each configuration combines the following:  * The EMM device policy controller (DPC) installed on the devices. * EMM policies enforced on the devices. * Metadata displayed on the device to help users during setup.  Customers can add as many configurations as they need. However, zero-touch enrollment works best when a customer sets a default configuration that&#39;s applied to any new devices the organization purchases.

  ## Attributes

  - companyName (String.t): Required. The name of the organization. Zero-touch enrollment shows this organization name to device users during device provisioning. Defaults to: `null`.
  - configurationId (String.t): Output only. The ID of the configuration. Assigned by the server. Defaults to: `null`.
  - configurationName (String.t): Required. A short name that describes the configuration&#39;s purpose. For example, _Sales team_ or _Temporary employees_. The zero-touch enrollment portal displays this name to IT admins. Defaults to: `null`.
  - contactEmail (String.t): Required. The email address that device users can contact to get help. Zero-touch enrollment shows this email address to device users before device provisioning. The value is validated on input. Defaults to: `null`.
  - contactPhone (String.t): Required. The telephone number that device users can call, using another device, to get help. Zero-touch enrollment shows this number to device users before device provisioning. Accepts numerals, spaces, the plus sign, hyphens, and parentheses. Defaults to: `null`.
  - customMessage (String.t): A message, containing one or two sentences, to help device users get help or give them more details about what’s happening to their device. Zero-touch enrollment shows this message before the device is provisioned. Defaults to: `null`.
  - dpcExtras (String.t): The JSON-formatted EMM provisioning extras that are passed to the DPC. Defaults to: `null`.
  - dpcResourcePath (String.t): Required. The resource name of the selected DPC (device policy controller) in the format &#x60;customers/[CUSTOMER_ID]/dpcs/*&#x60;. To list the supported DPCs, call &#x60;customers.dpcs.list&#x60;. Defaults to: `null`.
  - isDefault (boolean()): Required. Whether this is the default configuration that zero-touch enrollment applies to any new devices the organization purchases in the future. Only one customer configuration can be the default. Setting this value to &#x60;true&#x60;, changes the previous default configuration&#39;s &#x60;isDefault&#x60; value to &#x60;false&#x60;. Defaults to: `null`.
  - name (String.t): Output only. The API resource name in the format &#x60;customers/[CUSTOMER_ID]/configurations/[CONFIGURATION_ID]&#x60;. Assigned by the server. Defaults to: `null`.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :companyName => any(),
          :configurationId => any(),
          :configurationName => any(),
          :contactEmail => any(),
          :contactPhone => any(),
          :customMessage => any(),
          :dpcExtras => any(),
          :dpcResourcePath => any(),
          :isDefault => any(),
          :name => any()
        }

  field(:companyName)
  field(:configurationId)
  field(:configurationName)
  field(:contactEmail)
  field(:contactPhone)
  field(:customMessage)
  field(:dpcExtras)
  field(:dpcResourcePath)
  field(:isDefault)
  field(:name)
end

defimpl Poison.Decoder, for: GoogleApi.AndroidDeviceProvisioning.V1.Model.Configuration do
  def decode(value, options) do
    GoogleApi.AndroidDeviceProvisioning.V1.Model.Configuration.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.AndroidDeviceProvisioning.V1.Model.Configuration do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
